/*
 * This file contains the phone book database functions 
 * implementation
 */
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <ctype.h>
#include <time.h>

#include "contacts-db-pl.h"

static EBook *book; //Address book

/*
 * Function opens the system address book 
 */
gint db_open_pl (GError** error)
{
	book = e_book_new_system_addressbook(error);
	if (book == NULL)
		return -1;
	if (!e_book_open(book, FALSE, error)) {
		g_object_unref (book);
		return -1;
	}
	return 0;
}

/*
 * Function reads the address book and returns list of contacts that
 * matches name or returns NULL on error
 */
GList* db_get_phone_list (const gchar *name, GError** error) {
	GList* list = NULL; //List of contacts we shall return
	EBookQuery* query; //the resulting query

	if (book == NULL)
		return NULL;
	query = e_book_query_field_test (E_CONTACT_FULL_NAME, 
					 E_BOOK_QUERY_BEGINS_WITH, name);
	if (query == NULL)
		return NULL;
	if (!e_book_get_contacts(book, query, &list, error)) {
		e_book_query_unref (query);
		return NULL;
	}
	e_book_query_unref (query);
	return list;
}

/*
 * Function is called to close pl
 */
void db_close_pl(void)
{
	if (book != NULL)
		g_object_unref(book);
}

/*
 * Function returns a single contact with provided id
 */
EContact* getContact(const gchar* id)
{
	EContact* contact;
	if (book == NULL)
		return NULL;
	if (!e_book_get_contact(book, id, &contact, NULL))
		return NULL;
	else 
		return contact;
}

//eof
