/*
 * This file contains the main function and GUI 
 * functions of the application
 */
#include <string.h>
#include <locale.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <glib.h>
#include <gtk/gtk.h>

#include <hildon-widgets/hildon-program.h>
#include <hildon-widgets/hildon-banner.h>

#include <libosso.h>
#include <libebook/e-book.h>

#include "contacts-db-pl.h"
#include "gnokii.h"

/* D-BUS service definitions */
#define APPLICATION_DBUS_SERVICE "phonelink"

struct UICStruct {
	GtkListStore *storeNumber;
	GtkWidget *searchEntry;
	GtkWidget *treeNumber;
	GtkListStore *storeSearch;
	GtkWidget *treeSearch;
	GtkWidget* smsEdit; //SMS editor
	osso_context_t *osso_context;
};
typedef struct UICStruct * UIComponents;

static struct gn_statemachine * state;
static int smsError; //SMS error number

static int sendSMSToList (GSList *numberList, gchar * text);

static void  quit_event( gpointer   data,
                         guint      callback_action,
                         GtkWidget *menu_item	)
{
        gn_lib_phoneprofile_free(&state);
        gn_lib_library_free();
			
	UIComponents uic = (UIComponents) data;
	gtk_main_quit ();
	db_close_pl();
	if (uic->osso_context != NULL)
		osso_deinitialize(uic->osso_context);
	g_free(uic);
					
}

/* Description des colonnes*/
enum
{
   NAME_COLUMN,
   ID_COLUMN,
   N_COLUMNS
};

enum
{
   TYPE_COLUMN,
   NUMBER_COLUMN,
   N_COLUMNS_1
};

static void findNumber_cb(GtkTreeSelection *treeselection, gpointer data);

/* another callback */
static gboolean delete_event( GtkWidget *widget, GdkEvent *event, gpointer data )
{
	quit_event(data, 0, NULL);
	return TRUE;
}

static gchar * normalizeNumber( gchar *number) {

	if (!number) {
		return number;
	}

	gchar * normalizedNumber = (gchar *) g_malloc(strlen(number)+1);
	guint i = 0;
	guint j = 0;
	g_strstrip(number);
	if (number[0] == '+') {
		normalizedNumber[0] = '0';
		normalizedNumber[1] = '0';
		j = 2;
		i = 1;
	}

	for (; number[i]; i++) {
		if (g_ascii_isdigit(number[i]) || number[i] == '+') {
			normalizedNumber[j++] = number[i];
		}
	}
	normalizedNumber[j] = '\0';
	return normalizedNumber;
}

static gchar * normalizeNumber4Sip( gchar *number ) {

	gchar * num = normalizeNumber(number);
	gchar * res = g_strconcat("sip:", num, NULL); /*, "@freephonie.net" */
	g_free(num);
	return res;
}
	
static void search_cb (GtkButton *widget, gpointer data) {
	UIComponents uic = (UIComponents) data;
	EContact *p;
	GtkTreeIter iter;
	GList *entry;
	GtkTreeSelection *select;

	GList *list = db_get_phone_list(gtk_entry_get_text(GTK_ENTRY(uic->searchEntry)),
		NULL);
	
	if (list == NULL) {
		return;
	}

	gtk_list_store_clear(GTK_LIST_STORE(uic->storeSearch));
	for (entry=list; entry; entry=entry->next) {
		p = entry->data;
		if (p == NULL)
			continue;
		gtk_list_store_append(GTK_LIST_STORE(uic->storeSearch), &iter);
		gtk_list_store_set(GTK_LIST_STORE(uic->storeSearch), &iter, 
				   NAME_COLUMN,
				   g_strdup((gchar*)e_contact_get_const(p, 
									E_CONTACT_FULL_NAME)),
				   ID_COLUMN, 
				   g_strdup((gchar*)e_contact_get_const(p, E_CONTACT_UID)),
				   -1);

	}
	g_list_free(list);
	gtk_tree_view_columns_autosize(GTK_TREE_VIEW(uic->treeSearch));
	select = gtk_tree_view_get_selection(GTK_TREE_VIEW (uic->treeSearch));
	findNumber_cb(select, uic);
}

/*
 * Function is to be called to convert phone types to a fancy line
 */
static gchar* phone2str(GList* types)
{
	//Flags used for types
	typedef enum
	{
		PTF_HOME = 0x01,
		PTF_OFFICE = 0x02,
		PTF_CELL = 0x04
	} TypeFlag;
	struct { //structure describes types 
		gchar* name;  //name of the type
		TypeFlag flag; //corr. flag
	} typeTable[] = {
		{ "home", PTF_HOME },
		{ "cell", PTF_CELL },
		{ "work", PTF_OFFICE }
	};
	gint typeTableSize = sizeof(typeTable) / sizeof(typeTable[0]);
	struct { //fancy names and flags
		gint flag;
		gchar* name;
	} fancyNames [] = {
		{ PTF_HOME, "Home: " }, 
		{ PTF_OFFICE, "Office: " }, 
		{ PTF_CELL, "Mobile: " },
		{ PTF_HOME | PTF_CELL, "Home mobile: " },
		{ PTF_OFFICE | PTF_CELL, "Office mobile: " },
	};
	gint fancyNamesSixe = sizeof(fancyNames) / sizeof(fancyNames[0]);
	GList* idx; //list index
	GList* jdx; //list index
	gint typeId = 0; //The resulting type id
	int i; //just an index
	for (idx = types; idx; idx = idx->next) {
		EVCardAttributeParam* param = (EVCardAttributeParam*)idx->data;
		if (strcmp(e_vcard_attribute_param_get_name(param), "TYPE") == 0) {
			GList* vals = e_vcard_attribute_param_get_values(param);
			for (jdx = vals; jdx; jdx = jdx->next) {
				for (i = 0; i < typeTableSize; i++) {
					if (!strcmp(g_strdown((gchar*)jdx->data), 
						    typeTable[i].name)) {
						typeId |= typeTable[i].flag;
						break;
					}
				}
			}
		}
	}
	for (i = 0; i < fancyNamesSixe; i++) {
		if (typeId == fancyNames[i].flag)
			return g_strdup(fancyNames[i].name);
	}
	return g_strdup("Phone: ");
}

/*
 * Function fills up the phone list from contact information in contacts
 */
void fillPhoneList(UIComponents uic, EContact* contact)
{
	GtkTreeIter iter; //iterator in the list view
	EVCardAttribute* phone; //phone attribute
	GList* phoneTypes; //phone types
	GList* phoneNumbers; //phone numbers
	GList* attrList; //list of all the attributes
	GList* idx; //index to search attributes
	
////	e_vcard_dump_structure(E_VCARD(contact));
	attrList = e_vcard_get_attributes(E_VCARD(contact));
	if (attrList == NULL)
		return;
	//Let's seach for phones
	gtk_list_store_clear(GTK_LIST_STORE(uic->storeNumber));
	for (idx = attrList; idx; idx = idx->next) {
		phone = (EVCardAttribute*)idx->data;
		if (strcmp(e_vcard_attribute_get_name(idx->data), "TEL"))
			continue;
		phoneTypes = e_vcard_attribute_get_params(phone);
		phoneNumbers = e_vcard_attribute_get_values(phone);
		gtk_list_store_append(GTK_LIST_STORE(uic->storeNumber), &iter);
		gtk_list_store_set(GTK_LIST_STORE(uic->storeNumber), &iter, 
				   TYPE_COLUMN, phone2str(phoneTypes),
				   NUMBER_COLUMN, g_strdup((gchar*)phoneNumbers->data),
				   -1);

	}
}

/*
 * Function is called when a name is selected
 */
static void findNumber_cb(GtkTreeSelection *treeselection, gpointer data)
{
	UIComponents uic = (UIComponents) data;
	GtkTreeSelection *select;
	GList *list;
	gchar *id;
	GtkTreeModel *model;
	GtkTreeIter iter;
	EContact* contact; //contact to return

	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(uic->treeSearch));
	list = gtk_tree_selection_get_selected_rows(select, &model);
	if (list != NULL && 
	    gtk_tree_model_get_iter(model, &iter, (GtkTreePath *)list->data)) {
		gtk_tree_model_get(model, &iter, ID_COLUMN, &id, -1);
		if (id != NULL) {
////			g_print("id = '%s'\n", id);
			contact = getContact(id);
			if (contact != NULL) {
				fillPhoneList(uic, contact);
				g_object_unref(contact);
			}
			g_free(id);
		}
	}
	g_list_free(list);
}

static void dial  (GtkButton *widget, gpointer data, gboolean gsm);

static void dialSip_cb (GtkButton *widget, gpointer data) {
	dial(widget, data, FALSE);
}

static guint dialSip(gchar * number, gchar * message, UIComponents uic) {
	
/* initialise the connection */
	osso_rpc_t retval;
	osso_return_t ret;
	ret = osso_rpc_run(uic->osso_context,
			"com.nokia.gizmo",
			"/com/nokia/gizmo",
			"com.nokia.gizmo",
			"callto", 
			&retval,
			DBUS_TYPE_STRING, number,
			DBUS_TYPE_INVALID);
	osso_rpc_free_val(&retval);
	if (ret !=  OSSO_OK) {
		sprintf(message, "Can't dial Sip phone through DBus : %d", ret);
		return 1;
	}
	return 0;
}

static guint dialGsm(gchar *number, gchar *message) {

	guint ret = 0;

	gn_data gdata;
        gn_error error;
        gn_call_info call_info;
        int call_id;
				
	/* initialise the connection */
	if ((error = gn_lib_phone_open(state)) != GN_ERR_NONE) {
		g_print("Telephone interface init failed: %s Quitting.\n", 
			gn_error_print(error));
		return 1;
	}

	memset(&call_info, 0, sizeof(call_info));
	snprintf(call_info.number, sizeof(call_info.number), "%s", number);
	call_info.type = GN_CALL_Voice;
	call_info.send_number = GN_CALL_Default;
	gn_data_clear(&gdata);
	gdata.call_info = &call_info;
        if ((error = gn_call_dial(&call_id, &gdata, state)) != GN_ERR_NONE) {
		sprintf(message, "Can't dial the phone : %s", gn_error_print(error));
		ret = 2;
	} else {
		g_print("Dialled call, id: %d (lowlevel id: %d)\n", 
			call_id, call_info.call_id);
	}
	/* tear down the connection */
	gn_lib_phone_close(state);
	return ret;
}

static void dial_cb (GtkButton *widget, gpointer data) {
	dial(widget, data, TRUE);
}


static void dial(GtkButton *widget, gpointer data, gboolean gsm) {
	
	UIComponents uic = (UIComponents) data;
	
	GtkTreeSelection *select;
	GList *list;
	guint count;
	gchar *number;
	gchar *normalizedNumber;
	gchar text[200];
	GtkTreeIter iter;
	GtkTreeModel *model;

	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(uic->treeNumber));
	list = gtk_tree_selection_get_selected_rows(select, &model);

	count = g_slist_length((GSList *)list);
	g_print("%d selected numbers\n", count);
	if (count == 0) {
		// dial the content of the search Entry
		number = g_strdup(gtk_entry_get_text(GTK_ENTRY(uic->searchEntry)));
	} else if (count == 1) {
		if (gtk_tree_model_get_iter(model, &iter, (GtkTreePath *)list->data)) {
			gtk_tree_model_get(model, &iter, NUMBER_COLUMN, &number, -1);
		}
	} else {
		GtkWidget *dialog = 
			gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(widget))), 
					       GTK_DIALOG_DESTROY_WITH_PARENT, 
					       GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
					       "Select only one number to dial !");
		gtk_dialog_run (GTK_DIALOG(dialog));
		gtk_widget_destroy (dialog);
		g_list_free(list);
		return;
	}
	g_print("Number read = '%s'\n", number);
	if (number && number[0] != '\0') {
		normalizedNumber = normalizeNumber(number);
		if (normalizedNumber == NULL || normalizedNumber[0] == '\0') {
			sprintf(text, "Number %s is not Valid !\n", number);
			g_print(text);
			hildon_banner_show_information(GTK_WIDGET(gtk_widget_get_toplevel(uic->searchEntry)), 
						       NULL, text);
			g_free(normalizedNumber);
			g_free(number);
			g_list_free(list);
			return;
		}
		g_print("Number to dial = '%s'\n", normalizedNumber);
		sprintf(text, "Dialing %s ...", normalizedNumber);
		hildon_banner_show_information(GTK_WIDGET(gtk_widget_get_toplevel(uic->searchEntry)), 
					       NULL, text);
		guint ret = 0;
		if (gsm == TRUE) {
			ret = dialGsm(normalizedNumber, text);
		} else {
			normalizedNumber = normalizeNumber4Sip(normalizedNumber);
			ret = dialSip(normalizedNumber, text, uic);
		}
		if (ret != 0) {
			hildon_banner_show_information(GTK_WIDGET(gtk_widget_get_toplevel(uic->searchEntry)), 
						       NULL, text);
			g_free(normalizedNumber);
			g_free(number);
			g_list_free(list);
			return;
		}
		
		g_free(normalizedNumber);
	}
	g_free(number);
	g_list_free(list);
}

static void printData(gpointer data, gpointer data2) {
	g_print("%s; ", (gchar *) data);
}

static gchar * getSMSText(UIComponents uic) {
	gchar *text;
	GtkTextBuffer *buffer;
	GtkTextIter start, end;
	
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(uic->smsEdit));
	gtk_text_buffer_get_bounds(buffer, &start, &end);
	text = gtk_text_buffer_get_text(buffer, &start, &end, TRUE);
	return text;
}

static void sendSMS_cb (GtkButton *widget, gpointer data) {
	UIComponents uic = (UIComponents) data;

        GtkTreeSelection *select;
	GList *list, *listIter;
	gchar *number, *text, *normalizedNumber;
	GSList *numberList = NULL;
	GtkTreeIter iter;
	GtkTreeModel *model;

	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(uic->treeNumber));
	list = gtk_tree_selection_get_selected_rows(select, &model);

	for (listIter = list; listIter; listIter = listIter->next) {
		if (gtk_tree_model_get_iter(model, &iter, (GtkTreePath *)listIter->data)) {
			gtk_tree_model_get(model, &iter, NUMBER_COLUMN, &number, -1);
			normalizedNumber = normalizeNumber(number);
			numberList = g_slist_prepend(numberList, normalizedNumber);
		}
	}
	
	g_print("numbers : ");
	g_slist_foreach(numberList, printData, (gpointer) NULL);
	g_print("\n");
	text = getSMSText(uic);
	if (text) {
		hildon_banner_show_information(
			GTK_WIDGET(gtk_widget_get_toplevel(uic->searchEntry)), 
			NULL, "Sending SMS");
		g_print("%s\n", text);
		//Update everything before sending SMS
		while (gtk_events_pending())
			gtk_main_iteration();
		if (sendSMSToList(numberList, text) < 0)
			hildon_banner_show_information(
				GTK_WIDGET(gtk_widget_get_toplevel(uic->searchEntry)), 
				NULL, "SMS send failure");
		else
			hildon_banner_show_information(
				GTK_WIDGET(gtk_widget_get_toplevel(uic->searchEntry)), 
				NULL, "SMS send OK");
		g_free(text);
	}
	g_slist_free(numberList);
}

static void sendSMS (gpointer pnumber, gpointer ptext) {
	gchar *number = (gchar *)pnumber;
	gchar *text = (gchar *)ptext;
	gn_sms sms;
	gn_error error;
	gn_data data;

	gn_data_clear(&data);
	
	/* The maximum length of an uncompressed concatenated short message is
	 * 	   255 * 153 = 39015 default alphabet characters */
	int input_len, i, curpos = 0;

	input_len = GN_SMS_MAX_LENGTH;

	/* The memory is zeroed here */
	gn_sms_default_submit(&sms);

	memset(&sms.remote.number, 0, sizeof(sms.remote.number));
	strncpy(sms.remote.number, number, sizeof(sms.remote.number) - 1);
	if (sms.remote.number[0] == '+') {
		sms.remote.type = GN_GSM_NUMBER_International;
	} else {
		sms.remote.type = GN_GSM_NUMBER_Unknown;
	}
	
	data.message_center = calloc(1, sizeof(gn_sms_message_center));
	data.message_center->id = 1;
	if (gn_sm_functions(GN_OP_GetSMSCenter, &data, state) == GN_ERR_NONE) {
		strcpy(sms.smsc.number, data.message_center->smsc.number);
		sms.smsc.type = data.message_center->smsc.type;
	} else {
		g_print("Cannot read the SMSC number from your phone. \
If the sms send will fail, please use --smsc option explicitely giving the number.\n");
																			}
	free(data.message_center);

	if (!sms.smsc.type) sms.smsc.type = GN_GSM_NUMBER_Unknown;

	i = strlen(text);
	strncpy(sms.user_data[curpos].u.text, text, input_len);
	sms.user_data[curpos].u.text[i] = '\0';
	sms.user_data[curpos].length = i;
	if (sms.user_data[curpos].length < 1) {
		g_print("Empty message. Quitting.\n");
		g_free(number);
		return ;
	}
	sms.user_data[curpos].type = GN_SMS_DATA_Text;
	if (!gn_char_def_alphabet(sms.user_data[curpos].u.text)) {
		sms.dcs.u.general.alphabet = GN_SMS_DCS_UCS2;
	}
	sms.user_data[++curpos].type = GN_SMS_DATA_None;
	data.sms = &sms;

	/* Send the message. */
	error = gn_sms_send(&data, state);

	if (error == GN_ERR_NONE) {
		g_print("Send succeeded!\n");
	} else {
		g_print("SMS Send failed (%s)\n", gn_error_print(error));
		smsError = -1;
	}

	g_free(number);
}

static int sendSMSToList (GSList *numberList, gchar * text) {
	
	gn_error error;
	
	/* initialise the connection */
	if ((error = gn_lib_phone_open(state)) != GN_ERR_NONE) {
		g_print("Telephone interface init failed: %s Quitting.\n", 
			gn_error_print(error));
		return -1;
	}

	smsError = 0;
	g_slist_foreach (numberList, sendSMS, (gpointer)text);

	/* tear down the connection */
	gn_lib_phone_close(state);
	return smsError;
}

int main( int argc, char *argv[] )
{
	    /* GtkWidget is the storage type for widgets */
	GtkWidget *button;
	GtkWidget *searchButton;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkTreeSelection *select;
	GtkWidget *table;
	GtkWidget *scrolled_window;
	
	HildonProgram *app;
	HildonWindow *appview;
	
	GError* error= NULL; //the error code

	/* Initialize OSSO and install D-Bus handler */
	UIComponents uic = (UIComponents) g_malloc (sizeof(struct UICStruct));

	/* This is called in all GTK applications. Arguments are parsed
	 * from the command line and are returned to the application. */
	gtk_init (&argc, &argv);

	/* Create a new window */
        app = HILDON_PROGRAM( hildon_program_get_instance() );
        g_set_application_name(("PhoneLink" ));
	
        /* Create HildonAppView and set it to HildonApp */
        appview = HILDON_WINDOW(hildon_window_new());
        hildon_program_add_window( app, appview );


	/* Gnokii : read the config */
/*	if (gn_cfg_read(&bindir) < 0) {
		 hildon_banner_show_information(GTK_WIDGET(appview), NULL, 
		 "gnokii rc file reading problem\n");
	} else {
		if (!gn_cfg_phone_load("", state)) {
			 hildon_banner_show_information(GTK_WIDGET(appview), NULL, 
			 "gnokki rc file loading problem\n");
		}
	}
*/
	gn_error err;
	if ((err = gn_lib_phoneprofile_load(NULL, &state)) != GN_ERR_NONE) {
		fprintf(stderr, "%s\n", gn_error_print(err));
		hildon_banner_show_information(GTK_WIDGET(appview), NULL,  
					       gn_error_print(err));
	}

////	g_print("avant osso init\n");

	/* Initialize maemo application */
	uic->osso_context = osso_initialize(APPLICATION_DBUS_SERVICE, "0.0.1", TRUE, NULL);

////	g_print("apres osso_init\n");
	            
	/* Check that initialization was ok */
	if (uic->osso_context == NULL) {
		 hildon_banner_show_information(GTK_WIDGET(appview), NULL, 
						"osso_initialize error\n");
		return OSSO_ERROR;
	}
	if (db_open_pl(&error)) {
		hildon_banner_show_information(GTK_WIDGET(appview), 
					       NULL, "Database Access Problem\n");
		g_error_free(error);
		return OSSO_ERROR;
	}

	/* Here we just set a handler for delete_event that immediately
	*      * exits GTK. */
	g_signal_connect (G_OBJECT (appview), "delete_event",
		      G_CALLBACK (delete_event), (gpointer) uic);

	// la boite gnrale
	table = gtk_table_new(9, 5, TRUE);
	gtk_container_add (GTK_CONTAINER (appview), table);
	gtk_table_set_col_spacings(GTK_TABLE(table), 1);
	
	/* Add a field & 1 button */
	uic->searchEntry = gtk_entry_new();
	gtk_table_attach_defaults( GTK_TABLE(table), uic->searchEntry, 0, 4, 0, 1);

	/* Buttons */
	searchButton = gtk_button_new_with_label ("Search");
	GTK_WIDGET_SET_FLAGS(searchButton, GTK_CAN_DEFAULT);
	g_signal_connect (G_OBJECT (searchButton), "clicked",
	                  G_CALLBACK (search_cb), (gpointer) uic);
	gtk_table_attach_defaults(GTK_TABLE(table), searchButton, 4, 5, 0, 1);
	gtk_widget_grab_default(searchButton);

	// la liste des entres
	uic->storeSearch = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);

	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
				       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	
	uic->treeSearch = gtk_tree_view_new_with_model (GTK_TREE_MODEL (uic->storeSearch));
	
	GTK_WIDGET_UNSET_FLAGS(uic->treeSearch, GTK_CAN_FOCUS);
	gtk_container_add(GTK_CONTAINER(scrolled_window), uic->treeSearch);
	gtk_table_attach_defaults( GTK_TABLE(table), scrolled_window, 0, 2, 1, 9);

	//Set names
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Name", renderer, "text", 
							   NAME_COLUMN, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (uic->treeSearch), column);

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("ID", renderer, "text", 
							   ID_COLUMN, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (uic->treeSearch), column);
	gtk_tree_view_column_set_visible (column, FALSE);
	gtk_tree_view_set_search_column(GTK_TREE_VIEW (uic->treeSearch), NAME_COLUMN);
	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(
						     GTK_LIST_STORE(uic->storeSearch)), 
					     NAME_COLUMN, GTK_SORT_ASCENDING);
	/* Setup the selection handler */
	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (uic->treeSearch));
	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
	g_signal_connect (G_OBJECT (select), "changed",
	                  G_CALLBACK (findNumber_cb), (gpointer) uic);

	//Phone numbers
	uic->storeNumber = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);
	
	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
				       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		
	uic->treeNumber = gtk_tree_view_new_with_model (GTK_TREE_MODEL (uic->storeNumber));
	
	GTK_WIDGET_UNSET_FLAGS(uic->treeNumber, GTK_CAN_FOCUS);
	gtk_container_add(GTK_CONTAINER(scrolled_window), uic->treeNumber);
	gtk_table_attach_defaults( GTK_TABLE(table), scrolled_window, 2, 5, 1, 4);
	
        renderer = gtk_cell_renderer_text_new ();
        column = gtk_tree_view_column_new_with_attributes ("Type", renderer, "text", 
							   TYPE_COLUMN, NULL);
        gtk_tree_view_append_column (GTK_TREE_VIEW (uic->treeNumber), column);
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Number", renderer, "text", 
							   NUMBER_COLUMN, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (uic->treeNumber), column);
			
	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(uic->treeNumber));
	gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);

	//Set up the SMS editor
	uic->smsEdit = gtk_text_view_new();
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(uic->smsEdit), GTK_WRAP_WORD_CHAR);

	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
				       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(scrolled_window, GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(scrolled_window), uic->smsEdit);
	gtk_table_attach_defaults( GTK_TABLE(table), scrolled_window, 2, 5, 4, 8);
	
	//Set buttons
	button = gtk_button_new_with_label ("Dial Sip");
	g_signal_connect (G_OBJECT (button), "clicked",
	                  G_CALLBACK (dialSip_cb), (gpointer) uic);
	gtk_table_attach_defaults(GTK_TABLE(table), button, 2, 3, 8, 9);
			
	button = gtk_button_new_with_label ("Dial");
        g_signal_connect (G_OBJECT (button), "clicked",
                          G_CALLBACK (dial_cb), (gpointer) uic);
        gtk_table_attach_defaults(GTK_TABLE(table), button, 3, 4, 8, 9);
	
	button = gtk_button_new_with_label ("Send SMS");
	g_signal_connect (G_OBJECT (button), "clicked",
	                  G_CALLBACK (sendSMS_cb), (gpointer) uic);
	gtk_table_attach_defaults(GTK_TABLE(table), button, 4, 5, 8, 9);
	
	gtk_widget_show_all (GTK_WIDGET(appview));
	
	gtk_signal_emit_by_name(GTK_OBJECT(searchButton), "clicked");
	/* Rest in gtk_main and wait for the fun to begin! */
	gtk_main ();

	return OSSO_OK;
}

